@@ -2,48 +2,51 @@
 
 class XResultSet
 {
-    public:
-	long size;
-    	Xapian::docid * data;
+	public:
+		long size;
+		Xapian::docid * data;
 
-    XResultSet() { size=0; data=NULL; }
-    ~XResultSet() { if (size>0) { i_free(data); } }
+	XResultSet() { size=0; data=NULL; }
+	~XResultSet() { if (size>0) { i_free(data); } }
 
-    void add(Xapian::docid did)
-    {
-        if(data==NULL)
-        {
-            data=(Xapian::docid *)i_malloc(sizeof(Xapian::docid));
-        }
-        else
-        {
-            data=(Xapian::docid *)i_realloc(data,size*sizeof(Xapian::docid),(size+1)*sizeof(Xapian::docid));
-        }
-        data[size]=did;
-        size++;
-    }
+	void add(Xapian::docid did)
+	{
+		if(data==NULL)
+		{
+			data=(Xapian::docid *)i_malloc(sizeof(Xapian::docid));
+		}
+		else
+		{
+			data=(Xapian::docid *)i_realloc(data,size*sizeof(Xapian::docid),(size+1)*sizeof(Xapian::docid));
+		}
+		data[size]=did;
+		size++;
+	}
 };
-   
+
 class XQuerySet
 {
+	private:
+		icu::Transliterator *accentsConverter;
 	public:
-	char * header;
-	char * text;
-	XQuerySet ** qs;
-	bool global_and; // global
-	bool global_neg; // global
-	bool item_neg; // for the term
-	long qsize;
-	long limit;
+		char * header;
+		char * text;
+		XQuerySet ** qs;
+		bool global_and; // global
+		bool global_neg; // global
+		bool item_neg; // for the term
+		long qsize;
+		long limit;
 
 	XQuerySet()
 	{
 		qsize=0; qs=NULL;
-                limit=1;
+		limit=1;
 		global_and=true;
 		header=NULL;
-                text=NULL;
+		text=NULL;
 		global_neg=false;
+		accentsConverter=NULL;
 	}
 
 	XQuerySet(bool is_and, bool is_neg, long l)
@@ -55,84 +58,83 @@ class XQuerySet
 		text=NULL;
 		global_and=is_and;
 		global_neg=is_neg;
+		accentsConverter=NULL;
 	}
 
 	~XQuerySet()
-        {
-                if(text!=NULL) { i_free(text); text=NULL; }
-                if(header!=NULL) { i_free(header); header=NULL; }
+	{
+		if(text!=NULL) { i_free(text); text=NULL; }
+		if(header!=NULL) { i_free(header); header=NULL; }
 
-                for(long j=0;j<qsize;j++)
-                {
-                        delete(qs[j]);
-                }
-                if(qsize>0) i_free(qs);
-                qsize=0; qs=NULL;
-        }
+		for(long j=0;j<qsize;j++)
+		{
+			delete(qs[j]);
+		}
+		if(qsize>0) i_free(qs);
+		qsize=0; qs=NULL;
+		if(accentsConverter != NULL) delete(accentsConverter);
+	}
 
 	void add(const char * h,const char * t)
 	{
 		add(h,t,false);
 	}
 
-        void add(const char * h,const char * t, bool is_neg)
-        {
-                if(h==NULL) return;
-                if(t==NULL) return;
+	void add(const char * h,const char * t, bool is_neg)
+	{
+		if(h==NULL) return;
+		if(t==NULL) return;
 
-                icu::StringPiece sp_h(h);
-                icu::UnicodeString h2 = icu::UnicodeString::fromUTF8(sp_h);
+		icu::UnicodeString h2 = icu::UnicodeString::fromUTF8(icu::StringPiece(h));
+		icu::UnicodeString t2 = icu::UnicodeString::fromUTF8(icu::StringPiece(t));
 
-                icu::StringPiece sp_t(t);
-                icu::UnicodeString t2 = icu::UnicodeString::fromUTF8(sp_t);
-
-                add(&h2,&t2,is_neg);
-        }
+		add(&h2,&t2,is_neg);
+	}
 
 	void add(icu::UnicodeString *h, icu::UnicodeString *t, bool is_neg)
-        {
+	{
 		long i,j;
 		XQuerySet * q2;
 		icu::UnicodeString *r;
 
-	        t->findAndReplace("'"," ");
+		t->findAndReplace("'"," ");
 		t->findAndReplace("\""," ");
-                t->findAndReplace(":"," ");
-                t->findAndReplace(";"," ");
-                t->findAndReplace("\""," ");
-                t->findAndReplace("<"," ");
-                t->findAndReplace(">"," ");
-                t->findAndReplace("\n"," ");
-                t->findAndReplace("\r"," ");
+		t->findAndReplace(":"," ");
+		t->findAndReplace(";"," ");
+		t->findAndReplace("\""," ");
+		t->findAndReplace("<"," ");
+		t->findAndReplace(">"," ");
+		t->findAndReplace("\n"," ");
+		t->findAndReplace("\r"," ");
 		t->findAndReplace("@"," ");
 		t->findAndReplace("-","_");
 
 		h->trim();
 		t->trim();
-                h->toLower();
-                t->toLower();
+		h->toLower();
+		t->toLower();
 
 		if(h->length()<1) return;
 		if(t->length()<limit) return;
 
 		i = t->lastIndexOf(" ");
-                if(i>0)
-                {
-                        q2 = new XQuerySet(true,false,limit);
-                        while(i>0)
-                        {
-                                j = t->length();
-                                r = new icu::UnicodeString(*t,i+1,j-i-1);
-                                q2->add(h,r,false);
-                                delete(r);
-                                t->truncate(i);
+		if(i>0)
+		{
+			q2 = new XQuerySet(true,false,limit);
+			while(i>0)
+			{
+				j = t->length();
+				r = new icu::UnicodeString(*t,i+1,j-i-1);
+				q2->add(h,r,false);
+				delete(r);
+				t->truncate(i);
 				t->trim();
-                                i = t->lastIndexOf(" ");
-                        }
-                        q2->add(h,t,false);
-                        if(q2->count()>0) add(q2); else delete(q2);
-                        return;
-                }
+				i = t->lastIndexOf(" ");
+			}
+			q2->add(h,t,false);
+			if(q2->count()>0) add(q2); else delete(q2);
+			return;
+		}
 
 		i = t->indexOf(".");
 		if(i>=0)
@@ -146,17 +148,30 @@ class XQuerySet
 			t->findAndReplace("."," ");
 			t->trim();
 			q2->add(h,t,false);
-                	
+
 			if(q2->count()>0) add(q2); else delete(q2);
 			return;
 		}
-		
-                std::string tmp1;
-                h->toUTF8String(tmp1);
-                char * h2 = i_strdup(tmp1.c_str());
-                std::string tmp2;
-                t->toUTF8String(tmp2);
-                char * t2 = i_strdup(tmp2.c_str());
+
+		std::string tmp1;
+		h->toUTF8String(tmp1);
+		char * h2 = i_strdup(tmp1.c_str());
+
+		if(accentsConverter == NULL)
+		{
+			UErrorCode status = U_ZERO_ERROR;
+                	accentsConverter = icu::Transliterator::createInstance("NFD; [:M:] Remove; NFC", UTRANS_FORWARD, status);
+                	if(U_FAILURE(status))
+                	{
+                	        i_error("FTS Xapian: Can not allocate ICU translator (2)");
+                	        accentsConverter = NULL;
+                	}
+		}
+		if(accentsConverter != NULL) accentsConverter->transliterate(*t);
+
+		std::string tmp2;
+		t->toUTF8String(tmp2);
+		char * t2 = i_strdup(tmp2.c_str());
 
 		if(strcmp(h2,XAPIAN_WILDCARD)==0)
 		{
@@ -167,7 +182,7 @@ class XQuerySet
 			}
 			add(q2);
 			i_free(h2);
-                        i_free(t2);
+			i_free(t2);
 			return;
 		}
 
@@ -178,25 +193,25 @@ class XQuerySet
 		}
 		if(i>=HDRS_NB)
 		{
-			i_error("FTS Xapian: Unknown header (lookup) '%s'",h2);
+			if(verbose>1) i_error("FTS Xapian: Unknown header (lookup) '%s'",h2);
 			i_free(h2); i_free(t2);
 			return;
 		}
 
-                if(has(h2,t2,true))
-                {
-                        i_free(h2);
-                        i_free(t2);
-                        return;
-                }
+		if(has(h2,t2,true))
+		{
+			i_free(h2);
+			i_free(t2);
+			return;
+		}
 
-                if(text==NULL)
-                {
-                        text=t2;
-                        header=h2;
+		if(text==NULL)
+		{
+			text=t2;
+			header=h2;
 			item_neg=is_neg;
 			return;
-                }
+		}
 
 		q2 = new XQuerySet(global_and,is_neg,limit);
 		q2->add(h,t,false);
@@ -206,8 +221,8 @@ class XQuerySet
 	void add(XQuerySet *q2)
 	{
 		if(qsize<1)
-                {
-                        qs=(XQuerySet **)i_malloc(sizeof(XQuerySet*));
+		{
+			qs=(XQuerySet **)i_malloc(sizeof(XQuerySet*));
 		}
 		else
 		{
@@ -229,14 +244,14 @@ class XQuerySet
 		}
 		return false;
 	}
-	
+
 	int count()
 	{
 		int c=0;
 		if(text!=NULL) c=1;
 		c+=qsize;
 		return c;
-	} 
+	}
 
 	std::string get_string()
 	{
@@ -247,7 +262,7 @@ class XQuerySet
 		if(text!=NULL)
 		{
 			if(item_neg) s.append("NOT( ");
-			s.append(header); 
+			s.append(header);
 			//s.append(":");
 			s.append(":\"");
 			s.append(text);
@@ -275,11 +290,11 @@ class XQuerySet
 			{
 				s.append("(");
 				s.append(qs[i]->get_string());
-                                s.append(")");
+				s.append(")");
 			}
 			else s.append(qs[i]->get_string());
 		}
-		if(global_neg) 
+		if(global_neg)
 		{
 			s="NOT("+s+")";
 		}
@@ -287,25 +302,25 @@ class XQuerySet
 	}
 
 	Xapian::Query * get_query(Xapian::Database * db)
-        {
-                if(count()<1)
-                {
+	{
+		if(count()<1)
+		{
 			return new Xapian::Query(Xapian::Query::MatchNothing);
-                }
-		
+		}
+
 		Xapian::QueryParser * qp = new Xapian::QueryParser();
-		
+
 		for(int i=0; i< HDRS_NB; i++) qp->add_prefix(hdrs_emails[i], hdrs_xapian[i]);
 
 		char *s = i_strdup(get_string().c_str());
-	
+
 		qp->set_database(*db);
-	
+
 		Xapian::Query * q = new Xapian::Query(qp->parse_query(s,Xapian::QueryParser::FLAG_DEFAULT));// | Xapian::QueryParser::FLAG_PARTIAL));
-                
+
 		i_free(s);
-                delete(qp);
-                return q;
+		delete(qp);
+		return q;
 	}
 };
 
@@ -315,45 +330,47 @@ class XNGram
 		long partial,full,hardlimit;
 		const char * prefix;
 		bool onlyone;
+		icu::Transliterator *accentsConverter;
 
 	public:
 		char ** data;
 		long size,maxlength;
 		long memory;
-  
-        XNGram(long p, long f, const char * pre) 
-	{ 
-		partial=p; full=f; 
-		size=0; 
+
+	XNGram(long p, long f, const char * pre)
+	{
+		partial=p; full=f;
+		size=0;
 		memory=0;
 		maxlength=0;
-		data=NULL; 
+		data=NULL;
 		prefix=pre;
 		hardlimit=XAPIAN_TERM_SIZELIMIT-strlen(prefix);
 		onlyone=false;
 		if(strcmp(prefix,"XMID")==0) onlyone=true;
+		accentsConverter = NULL;
 	}
 
-        ~XNGram() 
-	{ 
+	~XNGram()
+	{
 		long i;
-		if (data != NULL) 
-		{ 
-			for(i=0;i<size;i++) 
-			{ 
-				i_free(data[i]); 
-			} 
-			i_free(data); 
+		if (data != NULL)
+		{
+			for(i=0;i<size;i++)
+			{
+				i_free(data[i]);
+			}
+			i_free(data);
 		}
 		data=NULL;
+		if(accentsConverter != NULL) delete(accentsConverter);
 	}
 
 	void add(const char * s)
-        {
-        	if(s==NULL) return;
+	{
+		if(s==NULL) return;
 
-		icu::StringPiece sp(s);
-		icu::UnicodeString d = icu::UnicodeString::fromUTF8(sp);
+		icu::UnicodeString d = icu::UnicodeString::fromUTF8(icu::StringPiece(s));
 		add(&d);
 	}
 
@@ -373,7 +390,7 @@ class XNGram
 		d->findAndReplace("\r"," ");
 		d->findAndReplace("@"," ");
 		d->findAndReplace("-","_");
-
+		
 		long i = d->indexOf(".");
 		if(i>=0)
 		{
@@ -395,16 +412,28 @@ class XNGram
 			d->truncate(i);
 			d->trim();
 		}
-	
+
 		long l = d->length();
 		if(l<partial) return;
 
+		if(accentsConverter == NULL)
+		{
+			UErrorCode status = U_ZERO_ERROR;
+			accentsConverter = icu::Transliterator::createInstance("NFD; [:M:] Remove; NFC", UTRANS_FORWARD, status);
+			if(U_FAILURE(status))
+			{
+				i_error("FTS Xapian: Can not allocate ICU translator (1)");
+				accentsConverter = NULL;
+			}
+		}
+		if(accentsConverter != NULL) accentsConverter->transliterate(*d);
+
 		if(onlyone)
 		{
 			add_stem(d);
 			return;
 		}
-	
+
 		for(i=0;i<=l-partial;i++)
 		{
 			for(long j=partial;(j+i<=l)&&(j<=full);j++)
@@ -416,7 +445,7 @@ class XNGram
 		}
 		if(l>full) add_stem(d);
 	}
-	
+
 	void add_stem(icu::UnicodeString *d)
 	{
 		d->trim();
@@ -434,15 +463,15 @@ class XNGram
 
 		char * s2 = i_strdup(s.c_str());
 		long p =0;
- 
-                if(size<1)
-                {
-                	data=(char **)i_malloc(sizeof(char*));
+
+		if(size<1)
+		{
+			data=(char **)i_malloc(sizeof(char*));
 			size=0;
 			p=0;
-                }
-                else
-                {
+		}
+		else
+		{
 			p=0;
 			while((p<size) && (strcmp(data[p],s2)<0))
 			{
@@ -460,14 +489,21 @@ class XNGram
 				data[i]=data[i-1];
 				i--;
 			}
-                }
+		}
 		if(l>maxlength) { maxlength=l; }
-                data[p]=s2;
-                size++;
+		data[p]=s2;
+		size++;
 		memory = memory + ((l+1) * sizeof(data[p][0]));
 	}
 };
 
+static long fts_backend_xapian_current_time()
+{
+	struct timeval tp;
+	gettimeofday(&tp, NULL);
+	return tp.tv_sec * 1000 + tp.tv_usec / 1000;
+}
+
 static long fts_backend_xapian_memory_used() // KB
 {
 	FILE* file = fopen("/proc/self/status", "r");
@@ -488,7 +524,7 @@ static long fts_backend_xapian_memory_used() // KB
 				fclose(file);
 				return atol(p);
 			}
-		}	
+		}
 		fclose(file);
 	}
 	return 0;
@@ -496,36 +532,36 @@ static long fts_backend_xapian_memory_used() // KB
 
 static long fts_backend_xapian_memory_free() // KB
 {
-        FILE* file = fopen("/proc/meminfo", "r");
-        int i;
-        char line[129];
-        const char* p;
+	FILE* file = fopen("/proc/meminfo", "r");
+	int i;
+	char line[129];
+	const char* p;
 
-        if(file != NULL)
-        {
-                while (fgets(line, 128, file) != NULL)
-                {
-                        if (strncmp(line, "MemFree:", 8) == 0)
-                        {
-                                i = strlen(line);
-                                p = line+8;
-                                while (*p <'0' || *p > '9') p++;
-                                line[i-3] = '\0';
-                                fclose(file);
-                                return atol(p);
-                        }
-                }
-                fclose(file);
-        }
-        return 0;
+	if(file != NULL)
+	{
+		while (fgets(line, 128, file) != NULL)
+		{
+			if (strncmp(line, "MemFree:", 8) == 0)
+			{
+				i = strlen(line);
+				p = line+8;
+				while (*p <'0' || *p > '9') p++;
+				line[i-3] = '\0';
+				fclose(file);
+				return atol(p);
+			}
+		}
+		fclose(file);
+	}
+	return 0;
 }
 
-static bool fts_backend_xapian_test_memory(struct xapian_fts_backend *backend)
+static bool fts_backend_xapian_test_memory(struct xapian_fts_backend *backend, long add)
 {
 	rlim_t limit;
 
-        restrict_get_process_size(&limit);
-        long m = long((long)limit / 1024.0); // vsz_limit of dovecot.conf
+	restrict_get_process_size(&limit);
+	long m = long((long)limit / 1024.0); // vsz_limit of dovecot.conf
 	long used = fts_backend_xapian_memory_used();
 	long fri = fts_backend_xapian_memory_free(); // Free RAM
 
@@ -534,14 +570,16 @@ static bool fts_backend_xapian_test_memory(struct xapian_fts_backend *backend)
 	if(backend->max_push < m2) backend->max_push=m2;
 	m2=backend->max_push;
 
-	if(m<1) 
+	add = long(add/1024.0);
+
+	if(m<1)
 	{
-		if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB (%ld pushes), Free = %ld MB, Estimated required = %ld MB",long(used/1024), backend->nb_pushes, long(fri/1024), long(m2/1024));
+		if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB (%ld pushes), Free = %ld MB, Additional data %ld KB, Estimated required = %ld MB",long(used/1024), backend->nb_pushes, long(fri/1024), add, long(m2/1024));
 		return ((fri>XAPIAN_MIN_RAM*1024)&&(fri>m2));
 	}
 	else
 	{
-		if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB (%ld%%) (%ld pushes), Limit = %ld MB, Free = %ld MB, Estimated required = %ld MB",long(used/1024),long(used*100.0/m),backend->nb_pushes,long(m/1024),long(fri/1024), long(m2/1024));
+		if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB (%ld%%) (%ld pushes), Limit = %ld MB, Free = %ld MB, Additional data %ld KB, Estimated required = %ld MB",long(used/1024),long(used*100.0/m),backend->nb_pushes,long(m/1024),long(fri/1024), add, long(m2/1024));
 		return ((fri>XAPIAN_MIN_RAM*1024)&&(m>(used+m2))&&(fri>m2));
 	}
 }
@@ -602,9 +640,7 @@ static void fts_backend_xapian_oldbox(struct xapian_fts_backend *backend)
 	if(backend->old_guid != NULL)
 	{
 		/* Performance calculator*/
-		struct timeval tp;
-		gettimeofday(&tp, NULL);
-		long dt = tp.tv_sec * 1000 + tp.tv_usec / 1000 - backend->perf_dt;
+		long dt = fts_backend_xapian_current_time() - backend->perf_dt;
 		double r=0;
 		if(dt>0)
 		{
@@ -612,12 +648,12 @@ static void fts_backend_xapian_oldbox(struct xapian_fts_backend *backend)
 			r=r/dt;
 		}
 		/* End Performance calculator*/
-                
+
 		if(verbose>0) i_info("FTS Xapian: Done indexing '%s' (%s) (%ld msgs in %ld ms, rate: %.1f)",backend->old_boxname, backend->old_guid,backend->perf_nb,dt,r);
 
 		i_free(backend->old_guid); backend->old_guid = NULL;
 		i_free(backend->old_boxname); backend->old_boxname = NULL;
-        }
+	}
 }
 
 static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const char * reason, long commit_time)
@@ -626,20 +662,22 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
 
 	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_release (%s)",reason);
 
+	if(commit_time<1) commit_time = fts_backend_xapian_current_time();
+
 	if(backend->dbw !=NULL)
-        {
+	{
 		try
-                {
-                        backend->dbw->commit();
-                        backend->dbw->close();
-                }
-                catch(Xapian::Error e)
-                {
-                        i_error("FTS Xapian: %s : %s - %s",reason,e.get_type(),e.get_error_string());
+		{
+			backend->dbw->commit();
+			backend->dbw->close();
+		}
+		catch(Xapian::Error e)
+		{
+			i_error("FTS Xapian: %s : %s - %s",reason,e.get_type(),e.get_error_string());
 			err=true;
-                }
-                delete(backend->dbw);
-                backend->dbw = NULL;
+		}
+		delete(backend->dbw);
+		backend->dbw = NULL;
 		backend->commit_updates = 0;
 		backend->commit_time = commit_time;
 	}
@@ -648,15 +686,15 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
 	{
 		if(verbose>0) i_info("FTS Xapian: Re-creating index database due to error");
 		try
-        	{
-                        Xapian::WritableDatabase * db = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OVERWRITE | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS);
-                        db->close();
-                        delete(db);
-                }
-                catch(Xapian::Error e)
-                {
-                        i_error("FTS Xapian: Can't re-create Xapian DB (%s) %s : %s - %s",backend->boxname,backend->db,e.get_type(),e.get_error_string());
-                }
+		{
+			Xapian::WritableDatabase * db = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OVERWRITE | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS);
+			db->close();
+			delete(db);
+		}
+		catch(Xapian::Error e)
+		{
+			i_error("FTS Xapian: Can't re-create Xapian DB (%s) %s : %s - %s",backend->boxname,backend->db,e.get_type(),e.get_error_string());
+		}
 	}
 
 	backend->nb_pushes=0;
@@ -664,10 +702,7 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
 
 	if(verbose>0)
 	{
-		struct timeval tp;
-        	gettimeofday(&tp, NULL);
-        	long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
-		i_info("FTS Xapian: Committed '%s' in %ld ms",reason,current_time - commit_time);
+		i_info("FTS Xapian: Committed '%s' in %ld ms",reason,fts_backend_xapian_current_time() - commit_time);
 	}
 }
 
@@ -675,7 +710,7 @@ XResultSet * fts_backend_xapian_query(Xapian::Database * dbx, XQuerySet * query,
 {
 	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_query");
 
-        XResultSet * set= new XResultSet();
+	XResultSet * set= new XResultSet();
 
 	try
 	{
@@ -714,9 +749,7 @@ static void fts_backend_xapian_do_expunge(const char *fpath)
 {
 	Xapian::WritableDatabase * dbw;
 
-	struct timeval tp;
-        gettimeofday(&tp, NULL);
-        long dt = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+	long dt = fts_backend_xapian_current_time();
 
 	try
 	{
@@ -744,9 +777,8 @@ static void fts_backend_xapian_do_expunge(const char *fpath)
 		delete(dbw);
 		return;
 	}
-	
+
 	Xapian::docid docid;
-	char * s;
 
 	long j=result->size;
 	if(verbose>0) i_info("FTS Xapian: Expunging '%s' : %ld to do",fpath,j);
@@ -772,31 +804,28 @@ static void fts_backend_xapian_do_expunge(const char *fpath)
 	dbw->commit();
 	dbw->close();
 	delete(dbw);
-	gettimeofday(&tp, NULL);
-	dt = tp.tv_sec * 1000 + tp.tv_usec / 1000 - dt;
+	
+	dt = fts_backend_xapian_current_time() - dt;
 	i_info("FTS Xapian: Expunging '%s' done in %.2f secs",fpath,dt/1000.0);
-}	
+}
 
 static int fts_backend_xapian_unset_box(struct xapian_fts_backend *backend)
 {
-	if(verbose>1) i_info("FTS Xapian: Unset box '%s' (%s)",backend->boxname,backend->guid);
+	if(verbose>0) i_info("FTS Xapian: Unset box '%s' (%s)",backend->boxname,backend->guid);
 
-	struct timeval tp;
-        gettimeofday(&tp, NULL);
-        long commit_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+	long commit_time = fts_backend_xapian_current_time();
 
 	fts_backend_xapian_oldbox(backend);
-
 	fts_backend_xapian_release(backend,"unset_box",commit_time);
 
-	if(backend->db != NULL) 
+	if(backend->db != NULL)
 	{
 		i_free(backend->db);
 		backend->db = NULL;
-		
+
 		i_free(backend->guid);
 		backend->guid = NULL;
-		
+
 		i_free(backend->boxname);
 		backend->boxname = NULL;
 	}
@@ -804,19 +833,47 @@ static int fts_backend_xapian_unset_box(struct xapian_fts_backend *backend)
 	return 0;
 }
 
+static int fts_backend_xapian_set_path(struct xapian_fts_backend *backend)
+{
+	struct mail_namespace * ns = backend->backend.ns;
+	if(ns->alias_for != NULL)
+	{
+		if(verbose>0) i_info("FTS Xapian: Switching namespace");
+		ns = ns->alias_for;
+	}
+
+	const char * path = mailbox_list_get_root_forced(ns->list, MAILBOX_LIST_PATH_TYPE_INDEX);
+
+	if(backend->path != NULL) i_free(backend->path);
+	backend->path = i_strconcat(path, "/" XAPIAN_FILE_PREFIX, NULL);
+
+	if(verbose>0) i_info("FTS Xapian: Index path = %s",backend->path);
+
+	struct stat sb;
+	if(!( (stat(backend->path, &sb)==0) && S_ISDIR(sb.st_mode)))
+	{
+		if (mailbox_list_mkdir_root(backend->backend.ns->list, backend->path, MAILBOX_LIST_PATH_TYPE_INDEX) < 0)
+		{
+			i_error("FTS Xapian: can not create '%s'",backend->path);
+			return -1;
+		}
+	}
+	return 0;
+}
+
 static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct mailbox *box)
 {
 	if (box == NULL)
 	{
 		if(backend->guid != NULL) fts_backend_xapian_unset_box(backend);
-		if(verbose>0) i_info("FTS Xapian: Box is empty");
-                return 0;
+		if(verbose>1) i_info("FTS Xapian: Box is empty");
+		return 0;
 	}
 
 	const char * mb;
 	fts_mailbox_get_guid(box, &mb );
 
-	if(verbose>1) i_info("FTX Xapian: Set box '%s' (%s)",box->name,mb);
+	if(verbose>0) i_info("FTS Xapian: Set box '%s' (%s)",box->name,mb);
 
 	if((mb == NULL) || (strlen(mb)<3))
 	{
@@ -832,14 +889,12 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 
 	if(backend->guid != NULL) fts_backend_xapian_unset_box(backend);
 
-        struct timeval tp;
-        long current_time;
+	if(fts_backend_xapian_set_path(backend)<0) return -1;
 
-        gettimeofday(&tp, NULL);
-        current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+	long current_time = fts_backend_xapian_current_time();
 
-        backend->commit_updates = 0;
-        backend->commit_time = current_time;
+	backend->commit_updates = 0;
+	backend->commit_time = current_time;
 	backend->guid = i_strdup(mb);
 	backend->boxname = i_strdup(box->name);
 	backend->db = i_strdup_printf("%s/db_%s",backend->path,mb);
@@ -858,69 +913,68 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 		catch(Xapian::Error e)
 		{
 			i_error("FTS Xapian: Can't create Xapian DB (%s) %s : %s - %s",backend->boxname,backend->db,e.get_type(),e.get_error_string());
-        	}
+		}
 	}
 	i_free(t);
 
+	/* Performance calculator*/
+	backend->perf_dt = current_time;
+	backend->perf_uid=0;
+	backend->perf_nb=0;
+	backend->perf_pt=0;
+	/* End Performance calculator*/
 
-        /* Performance calculator*/
-        backend->perf_dt = current_time;
-        backend->perf_uid=0;
-        backend->perf_nb=0;
-        backend->perf_pt=0;
-        /* End Performance calculator*/
-
-        return 0;
+	return 0;
 }
 
 static void fts_backend_xapian_build_qs(XQuerySet * qs, struct mail_search_arg *a)
 {
-        const char * hdr;
+	const char * hdr;
 
-        while(a != NULL)
-        {
-		switch (a->type) 
+	while(a != NULL)
+	{
+		switch (a->type)
 		{
-        		case SEARCH_TEXT: 
-        		case SEARCH_BODY: 
-        		case SEARCH_HEADER:
-        		case SEARCH_HEADER_ADDRESS:
-        		case SEARCH_HEADER_COMPRESS_LWSP: break;
-        		default: a = a->next; continue;
-        	}
-		
+			case SEARCH_TEXT:
+			case SEARCH_BODY:
+			case SEARCH_HEADER:
+			case SEARCH_HEADER_ADDRESS:
+			case SEARCH_HEADER_COMPRESS_LWSP: break;
+			default: a = a->next; continue;
+		}
+
 		if((a->hdr_field_name == NULL)||(strlen(a->hdr_field_name)<1))
-                {
-                        if(a->type == SEARCH_BODY)
-                        {
-                                hdr="body";
-                        }
-                        else
-                        {
-                                hdr=XAPIAN_WILDCARD;
-                        }
-                }
-                else
-                {
-                        hdr=a->hdr_field_name;
-                }
-                if((a->value.str == NULL) || (strlen(a->value.str)<1))
-                {
-                        XQuerySet * q2 = new XQuerySet(false,a->match_not,qs->limit);
-                        fts_backend_xapian_build_qs(q2,a->value.subargs);
-                        if(q2->count()>0)
-                        {
-                                qs->add(q2);
-                        }
-                        else
-                        {
-                                delete(q2);
+		{
+			if(a->type == SEARCH_BODY)
+			{
+				hdr="body";
+			}
+			else
+			{
+				hdr=XAPIAN_WILDCARD;
+			}
+		}
+		else
+		{
+			hdr=a->hdr_field_name;
+		}
+		if((a->value.str == NULL) || (strlen(a->value.str)<1))
+		{
+			XQuerySet * q2 = new XQuerySet(false,a->match_not,qs->limit);
+			fts_backend_xapian_build_qs(q2,a->value.subargs);
+			if(q2->count()>0)
+			{
+				qs->add(q2);
+			}
+			else
+			{
+				delete(q2);
 			}
 		}
 		else
 		{
 			long i=0,j=strlen(hdr);
-       			std::string f2;
+			std::string f2;
 			while(i<j)
 			{
 				if((hdr[i]>' ') && (hdr[i]!='"') && (hdr[i]!='\'') && (hdr[i]!='-'))
@@ -928,29 +982,29 @@ static void fts_backend_xapian_build_qs(XQuerySet * qs, struct mail_search_arg *
 					f2+=tolower(hdr[i]);
 				}
 				i++;
-        		}
-        		char * h = i_strdup(f2.c_str());
-                        qs->add(h,a->value.str,a->match_not);
+			}
+			char * h = i_strdup(f2.c_str());
+			qs->add(h,a->value.str,a->match_not);
 			i_free(h);
-                }
-                a->match_always=true;
+		}
+		a->match_always=true;
 		a = a->next;
-        }
+	}
 }
 
 bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid, const char* field, icu::UnicodeString* data)
 {
 	bool ok=true;
 
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_index_hdr");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_index_hdr");
 
 	Xapian::WritableDatabase * dbx = backend->dbw;
 	long p = backend->partial;
 	long f = backend->full;
 
-	if(data->length()<p) { return true; }
+	if(data->length()<p) return true;
 
-	if(strlen(field)<1) { return true; }
+	if(strlen(field)<1) return true;
 
 	long i=0;
 	while((i<HDRS_NB) && (strcmp(field,hdrs_emails[i])!=0))
@@ -962,25 +1016,25 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 	const char * h = hdrs_xapian[i];
 
 	XQuerySet * xq = new XQuerySet();
-        char *u = i_strdup_printf("%d",uid);
-        xq->add("uid",u);
+	char *u = i_strdup_printf("%d",uid);
+	xq->add("uid",u);
 	i_free(u);
 
-        XResultSet * result=fts_backend_xapian_query(dbx,xq,1);
+	XResultSet * result=fts_backend_xapian_query(dbx,xq,1);
 
 	Xapian::docid docid;
 	Xapian::Document * doc = NULL;
 	try
 	{
 		if(result->size<1)
-       		{
+		{
 			doc = new Xapian::Document();
 			doc->add_value(1,Xapian::sortable_serialise(uid));
 			u = i_strdup_printf("Q%d",uid);
 			doc->add_term(u);
 			docid=dbx->add_document(*doc);
 			i_free(u);
-        	}
+		}
 		else
 		{
 			docid=result->data[0];
@@ -993,7 +1047,7 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 		if(doc!=NULL) delete(doc);
 		ok=false;
 	}
-		
+
 	delete(result);
 	delete(xq);
 
@@ -1001,10 +1055,10 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 
 	XNGram * ngram = new XNGram(p,f,h);
 	ngram->add(data);
-	
-	if(verbose>0) 
-	{ 
-		i_info("FTS Xapian: Ngram(%s) -> %ld items (total %ld KB)",h,ngram->size, ngram->memory/1024); 
+
+	if(verbose>0)
+	{
+		i_info("FTS Xapian: Ngram(%s) -> %ld items (total %ld KB)",h,ngram->size, ngram->memory/1024);
 	}
 
 	for(i=0;i<ngram->size;i++)
@@ -1031,7 +1085,7 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 		}
 		catch (std::bad_alloc& ba)
 		{
-			i_error("FTS Xapian: Memory error '%s'",ba.what());
+			i_info("FTS Xapian: Memory too low (hdr) '%s'",ba.what());
 			ok = false;
 		}
 	}
@@ -1045,13 +1099,13 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 {
 	bool ok = true;
 
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_index_text");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_index_text");
 
 	Xapian::WritableDatabase * dbx = backend->dbw;
-        long p = backend->partial;
-        long f = backend->full;
+	long p = backend->partial;
+	long f = backend->full;
 
-	if(data->length()<p) { return true; }
+	if(data->length()<p) return true;
 
 	XQuerySet * xq = new XQuerySet();
 
@@ -1066,12 +1120,12 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	try
 	{
 		if(result->size<1)
-        	{
+		{
 			doc = new Xapian::Document();
 			doc->add_value(1,Xapian::sortable_serialise(uid));
 			u = t_strdup_printf("Q%d",uid);
 			doc->add_term(u);
-                	docid=dbx->add_document(*doc);
+			docid=dbx->add_document(*doc);
 		}
 		else
 		{
@@ -1084,7 +1138,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 		i_error("FTS Xapian: fts_backend_xapian_index_text : %s - %s",e.get_type(),e.get_error_string());
 		if(doc!=NULL) delete(doc);
 		ok=false;
-        }
+	}
 
 	delete(result);
 	delete(xq);
@@ -1098,7 +1152,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	termgenerator->set_document(*doc2);
 
 	const char * h;
-	if(strcmp(field,"subject")==0) 
+	if(strcmp(field,"subject")==0)
 	{
 		h="S";
 	}
@@ -1110,7 +1164,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	data->toUTF8String(s);
 	termgenerator->set_stemming_strategy(Xapian::TermGenerator::STEM_NONE);
 	termgenerator->index_text_without_positions(s, 1, h);
-	
+
 	long l = strlen(h);
 	long n = doc2->termlist_count();
 	Xapian::TermIterator * ti = new Xapian::TermIterator(doc2->termlist_begin());
@@ -1120,7 +1174,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	while(n>0)
 	{
 		s = *(*ti);
-		c=s.c_str();	
+		c=s.c_str();
 		if(strncmp(c,h,l)==0)
 		{
 			ngram->add(c+l);
@@ -1135,8 +1189,8 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	for(n=0;n<ngram->size;n++)
 	{
 		snprintf(t,ngram->maxlength+6,"%s%s",h,ngram->data[n]);
-                try
-                {
+		try
+		{
 			doc->add_term(t);
 		}
 		catch(Xapian::Error e)
@@ -1151,7 +1205,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	delete(termgenerator);
 	delete(doc2);
 
-	if(ok) 
+	if(ok)
 	{
 		try
 		{
@@ -1159,10 +1213,10 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 		}
 		catch (std::bad_alloc& ba)
 		{
-			i_error("FTS Xapian: Memory error '%s'",ba.what());
+			i_info("FTS Xapian: Memory too low (text) '%s'",ba.what());
 			ok = false;
 		}
-	}	
+	}
 
 	delete(doc);
 
